/*
	JkMemoBoard.cpp

	Author	: Kang Jul Ki (Master 2nd)
	E-mail	: jkkang@sel.chonbuk.ac.kr
	Homepage: http://sel.chonbuk.ac.kr/~jkkang/
	Date	: May 1, 2000
	Class	: Software Engineering Lab
		  Dept. of Electronic Engineering
		  Chonbuk National Univ.	

	Compiler: Visual C++ 6.0 & gcc 2.7.2
	Tab Size: 8
	
	Copyright (c) 2000, Kang Jul Ki
*/

#include "JkMemoBoard.h"
#include <stdlib.h>	/* for the malloc()	*/

/**
 * Construtor.
 *
 * Arg:
 *	@pJkMemoMain: The Pointer of the Parent's class
 */
JkMemoBoard::JkMemoBoard(JkMemoMain* pJkMemoMain)
{
	m_pJkMemoMain	= pJkMemoMain;
	m_pDbArticle	= new DbArticle;

	m_iTotalArticle = 0;
	sp		= 0;
}

/**
 * Destructor.
 */
JkMemoBoard::~JkMemoBoard()
{
	delete m_pDbArticle;
}

/**
 * Show List
 *
 * Arg: void
 * Ret: none
*/
void JkMemoBoard::ShowList(void)
{
	int	iTotal = 0;
	char	szPath[0xff] = "";

	/**
	 * Open the DB
	*/
	sprintf(szPath, "./db/%s.gdbm", m_pJkMemoMain->GetDbName());
	DBOpen(szPath, GDBM_READER);

	m_pJkMemoMain->m_pSysConfig->iLastNum = DBiFetch("SysConfig.iLastNum");

	iTotal = MainIndex();

	/**
	 * Show List
	*/
	PrintIndex(iTotal);

	DBClose();
}	

/**
 * Main Index.
 *
 * Arg: void
 * Ret: 
 *	@iTotalArticle: the Total Number.
*/
int JkMemoBoard::MainIndex(void)
{
	int	iTotalArticle;

	iTotalArticle = GetIndex(m_pJkMemoMain->GetLastNum());

	if (iTotalArticle == -1)
	{
		qError("There is a fatal error in the DB(%s)<br>You should call the admin.", m_pJkMemoMain->GetDbName());
	}

	AdjustIndex();
	ThreadIndex();

	return	iTotalArticle;
}

/**
 * Print JkMemo Article
 *
 * Arg:
 *	@iTotal: The Total Number of the Article
 * Ret: none
*/
void JkMemoBoard::PrintIndex(int iTotal)
{
	int	i, j;
	int	iStartNum, iEndNum, iPages;
	int	iStartPage, iEndPage;
	int	iStatus, iVirtualNum;
	int	iArticlePerPage = m_pJkMemoMain->GetArticlePerPage();
	int	iPageLinkNum    = 10;

	DbList	head;
	char	szStatusMsg[0xff] = "";
	char	szTitle[0xff] = "";
	char	*szBgColor[] = { "#fafafa", "#f7f7f7", "#f4f4f4", "#f1f1f1",
				"#eeeeee", "#ebebeb", "#e8e8e8", "#e5e5e5",
				"#e2e2e2", "#dfdfdf", "#dcdcdc", "#d9d9d9",
				"#d6d6d6", "#d3d3d3", "#d0d0d0", "#cdcdcd" };
	/**
	 * Calculate the number of pages.
	*/
	if (iTotal == 0)
	{
		iPages = 0;
	}
	else
	{
		int pof;

		if ((iTotal % iArticlePerPage) == 0)	pof = 0;
		else					pof = 1;

		iPages = (iTotal / iArticlePerPage) + pof;
	}

	/**
	 * Calculate the raingement of the link
	*/
	if (iPages == 0)
	{
		iStartPage = iEndPage = 0;
	}
	else
	{
		int pof;

		if ((m_pJkMemoMain->GetPageNum() % iPageLinkNum) == 0)
		{	
			pof = 0;
		}

		else
		{
			pof = 1;
		}

		iEndPage = ((m_pJkMemoMain->GetPageNum() / iPageLinkNum) + pof) * iPageLinkNum;
		iStartPage = iEndPage - (iPageLinkNum - 1);
	}

	/**
	 *   
	*/
	if (m_pJkMemoMain->GetPageNum() > iPages)
	{
		m_pJkMemoMain->m_pDb->iPage = iPages;
	}

	/**
	 *  Index ȣ 
	*/
	if (m_pJkMemoMain->GetPageNum() == 0)
	{
		iStartNum = iEndNum = 0;
	}
	else
	{
		iStartNum = (m_pJkMemoMain->GetPageNum() - 1) * iArticlePerPage;
		iEndNum   = iStartNum + (iArticlePerPage - 1);
	}

	/**
	 * Print Header
	*/
	sprintf(szTitle, "JK ޸ Ver %s - Developed by Kang Jul Ki", m_pJkMemoMain->GetBuildInfo());

	m_pJkMemoMain->SetRefreshValue(300);
	m_pJkMemoMain->HeadHtml(szTitle);

	printf("<script language='JavaScript'>\n");
	printf("  function AdminWindow(url)\n");
	printf("  {\n");
	printf("     window.open(url, 'AdminWin', 'width=400, height=250, resizable=no, scrollbars=no, status=0');\n");
	printf("  }\n\n");
	printf("  function CheckForm()\n");
	printf("  {\n");
	printf("     var form = document.forms[0];\n\n");
	printf("     if (IsEmpty(form.Name.value) || form.Name.value == form.Name.defaultValue) {\n");
	printf("        alert(\"̸ Է ϼ\");\n");
	printf("        form.Name.focus();\n");
	printf("        return false;\n");
	printf("     }\n");
	printf("     if (IsEmpty(form.Title.value) || form.Title.value == form.Title.defaultValue) {\n");
	printf("        alert(\" Էϼ.\");\n");
	printf("        form.Title.focus();\n");
	printf("        return false;\n");
	printf("     }\n\n");
	printf("     function IsEmpty(val) {\n");
	printf("        for (var i=0; i<val.length; i++) {\n");
	printf("           if (val.substring(i,i+1) != \" \")\n");
	printf("              return false;\n");
	printf("        }\n");
	printf("        return true;\n");
	printf("    }\n");
	printf("  }\n\n");

	printf("  function OnEnter(field) {\n");
	printf("     field.className = 'fo';\n");
	printf("     if (field.value == field.defaultValue) {\n");
	printf("        field.value = '';\n");
	printf("     }\n");
	printf("  }\n\n");
	printf("  function OnExit(field) {\n");
	printf("     field.className = 'bl';\n");
	printf("     if (field.value == '') {\n");
	printf("        field.value = field.defaultValue;\n");
	printf("     }\n");
	printf("  }\n\n");
	printf("</script>\n\n");
	
	printf("<!-- DB Information -->\n");
	printf("<table border=0 cellpadding=0 cellspacing=0 width='%d'>\n", m_pJkMemoMain->GetWidth());
	printf("  <tr>\n");
	printf("    <td align=left>\n");
	printf("      <a href=%s?dbName=%s&dbMode=AdminLogin><font size=1>Admin</font></a>\n", qCGIname(), m_pJkMemoMain->GetDbName());
	printf("    </td>\n");
	printf("    <td align=right>\n");
	printf("      <font size=1><font face=''>ù湮</font> : <font color=darkgreen>%s</font>, \n", qitocomma(m_pJkMemoMain->GetTodayCount()));
	printf("      <font size=1><font face=''>ѹ湮</font> : <font color=darkgreen>%s</font>, \n", qitocomma(m_pJkMemoMain->GetTotalCount()));
	printf("      <font face=''>ʹȣ</font> : <font color=red>%d</font> / <font color=darkblue>%d</font></font>\n", m_pJkMemoMain->GetPageNum(), iPages);
	printf("    </td>\n");
	printf("  </tr>\n");
	printf("</table>\n\n");
	
	printf("<table border=0 cellpadding=0 cellspacing=1 bgcolor='#000000' width='%d'>\n", m_pJkMemoMain->GetWidth());
	printf("  <tr bgcolor='#717d7d'><td>\n");
	printf("<table border=0 cellpadding=1 cellspacing=0 bgcolor=white width='100%%' height='100%%'>\n");
	printf("  <tr height=22 bgcolor='#717d7d'>\n");
	printf("    <td align=center colspan=4><font color='#ffffff'><marquee behavior=alternate><b>%s</b></marquee></font></td>\n", m_pJkMemoMain->GetAd());
	printf("  </tr>\n");

	iVirtualNum = iTotal - ((m_pJkMemoMain->GetPageNum()-1)*iArticlePerPage);

	for (i=iStartNum, j=0; (i<=iEndNum) && (tIndex[i].num != 0); i++, iVirtualNum--)
	{
		iStatus = GetArticleInfo(tIndex[i].num, &head);

		printf("  <tr height=19 valign=top align=center bgcolor='%s' onMouseOver=this.style.backgroundColor=/*'#e5e5e5'*/'#ece5c6' onMouseOut=this.style.backgroundColor='%s'>\n", szBgColor[j], szBgColor[j++]);
		printf("    <td align=right><p title=' մϴ.'>&nbsp;&nbsp;<a href=%s?dbName=%s&dbMode=Remove&dbNum=%d>%d</a></p></td>\n", qCGIname(), m_pJkMemoMain->GetDbName(), head.iNum, iVirtualNum);

		if (strcmp(head.pszEMail, "") == 0)
		{
			printf("    <td nowrap>&nbsp;%s&nbsp;</td>\n", head.pszName);
		}
		else
		{
			printf("    <td nowrap><p title='%s'><a href=mailto:%s>&nbsp;%s&nbsp;</a></p></td>\n", head.pszEMail, head.pszEMail, head.pszName);
		}

		printf("    <td align=left><p title='%s  ۼǾϴ.'>%s</p></td>\n", head.pszIpAdd, head.pszTitle);
		printf("    <td><p title='ۼð: %s'>%s&nbsp;</a></td>\n", head.pszTime, head.pszDate);

		printf("  </tr>\n");

		FreeArticleInfo(&head);
	}

	printf("  <form method='post' action='%s' onSubmit=\"return CheckForm()\">\n", qCGIname());
	printf("    <input type=hidden name=dbName value='%s'>\n", m_pJkMemoMain->GetDbName());
	printf("    <input type=hidden name=dbMode value='Write'>\n\n");

	printf("  <tr bgcolor='%s'>\n", szBgColor[j]);
	printf("    <td colspan=4 align=center>\n");
	printf("      <input type=text name=Name size=6 maxlength=15 value='̸' onblur='OnExit(this)' onfocus='OnEnter(this)' class=editbox>\n");
	printf("      <input type=text name=EMail size=15 maxlength=40 value='E-Mail' onblur='OnExit(this)' onfocus='OnEnter(this)' class=editbox>\n");
	printf("      <input type=text name=Title size=%d maxlength=300 value='޼ Էϼ' onblur='OnExit(this)' onfocus='OnEnter(this)' class=editbox>\n", m_pJkMemoMain->GetWidth()/13);
	printf("      <input type=submit value='' class=button>\n");
	printf("    </td>\n");
	printf("  </tr>\n");
	printf("  </form>\n");
	printf("</table>\n");

	/**
	 * Page Link
	*/
	printf("<table border=0 cellpadding=1 cellspacing=0 bgcolor=white width='100%%' height='100%%'>\n");
	printf("  <tr bgcolor=#fef3eb>\n");
	printf("<!-- Page Link -->\n");
	printf("  <tr>\n");
	printf("    <td align=center>\n");
	printf("      <font face=Arial size=1>\n");

	if (iStartPage > 1)
	{
		strcpy(szStatusMsg, "ù ϴ.");
		printf("        <a href=%s?dbName=%s&dbMode=Board&dbPage=%d onMouseOver=\"window.status=('%s'); return true;\">[1]</a>\n", qCGIname(), m_pJkMemoMain->GetDbName(), 1, szStatusMsg);

		strcpy(szStatusMsg, " ϴ.");
		printf("        <a href=%s?dbName=%s&dbMode=Board&dbPage=%d onMouseOver=\"window.status=('%s'); return true;\">[Prev]</a> -\n", qCGIname(), m_pJkMemoMain->GetDbName(), iStartPage-1, szStatusMsg);
	}

	for (i=iStartPage; (i<=iEndPage) && (i<=iPages) && (i>0); i++)
	{
		if (i == m_pJkMemoMain->GetPageNum())
		{
			printf("      [<font color=red>%d</font>]\n", i);
		}
		else
		{
			sprintf(szStatusMsg, "%d  ϴ.", i);
			printf("      <a href=%s?dbName=%s&dbMode=Board&dbPage=%d onMouseOver=\"window.status=('%s'); return true;\">[%d]</a>\n", qCGIname(), m_pJkMemoMain->GetDbName(), i, szStatusMsg, i);
		}
	}

	if (iEndPage < iPages)
	{
		strcpy(szStatusMsg, " ϴ.");
		printf("      - <a href=%s?dbName=%s&dbMode=Board&dbPage=%d onMouseOver=\"window.status=('%s'); return true;\">[Next]</a>\n", qCGIname(), m_pJkMemoMain->GetDbName(), iEndPage+1, szStatusMsg);
	
		if (iEndPage+1 < iPages)
		{
			strcpy(szStatusMsg, " ϴ.");
			printf("      <a href=%s?dbName=%s&dbMode=Board&dbPage=%d onMouseOver=\"window.status=('%s'); return true;\">[%d]</a>\n", qCGIname(), m_pJkMemoMain->GetDbName(), iPages, szStatusMsg, iPages);
		}
	}

	printf("      </font>\n");
	printf("    </td>\n");
	printf("    <td align=center width='16%%'>\n");

	/**
	 * Prev
	*/
	if (m_pJkMemoMain->GetPageNum() > 1)
	{
		strcpy(szStatusMsg, " ");
		printf("      <a href=%s?dbName=%s&dbMode=Board&dbPage=%d onMouseOver=\"window.status=('%s'); return true;\"><font size=1></font></a>\n", qCGIname(), m_pJkMemoMain->GetDbName(), m_pJkMemoMain->GetPageNum()-1, szStatusMsg);
	}
	else
	{
		printf("      <font size=1 color=darkgray></font>\n");
	}

	/**
	 * Next
	*/
	if (m_pJkMemoMain->GetPageNum() < iPages)
	{
		strcpy(szStatusMsg, " ");
		printf("      <a href=%s?dbName=%s&dbMode=Board&dbPage=%d onMouseOver=\"window.status=('%s'); return true;\"><font size=1></font></a>\n", qCGIname(), m_pJkMemoMain->GetDbName(), m_pJkMemoMain->GetPageNum()+1, szStatusMsg);
	}
	else
	{
		printf("      <font size=1 color=darkgray></font>\n");
	}

	printf("    </td>\n");
	printf("  </tr>\n");
	printf("  <tr>\n");
	printf("    <td align=right colspan=4>\n");
	printf("      <p title=\" Ȩ\"><font face='' size=1>Copyleft (c) 2000, JK ޸ Ver %s Programmed by <a href=http://sel.chonbuk.ac.kr/~jkkang/ target=_blank>Kang Jul Ki</a>&nbsp;</font></p>\n", m_pJkMemoMain->GetBuildInfo());
	printf("    </td>\n");
	printf("  </tr>\n");

	printf("</table>\n\n");
	printf("  </td></tr>\n");
	printf("</table>\n\n");

	m_pJkMemoMain->TailHtml();
}

/**
 * Get Index
 *
 * Arg:
 *	@iNum: The Index
 * Ret:
 *	@j: The Total Number of the Article
*/
int JkMemoBoard::GetIndex(int iNum)
{
	int	i, j;
	char*	pStr;

	if (iNum < 0)	return	-1;

	if ((dIndex = (D_INDEX *)malloc(sizeof(D_INDEX)*(iNum+1))) == NULL)
	{
		qError("JkMemory allocation error.");
	}

	if ((tIndex = (T_INDEX *)malloc(sizeof(T_INDEX)*(iNum+1))) == NULL)
	{
		qError("JkMemory allocation error.");
	}

	/**
	 * DB Fetch.
	*/
	for (i=iNum, j=0; i>0; i--)
	{
		pStr = DBFetch("%d.iThread", i);
	
		/**
		 * If there is no iThread, assume that the article was deleted.
		*/
		if (pStr != NULL)
		{
			m_iTotalArticle++;

			dIndex[j].num = i;
			dIndex[j].thread = atoi(pStr);
			j++;

			free(pStr);
		}
	}

	dIndex[j].num = dIndex[j].thread = 0;

	return	j;
}

/**
 * Indexing by the Date
 *
 * Arg: void
 * Ret: none
*/
void JkMemoBoard::DateIndex(void)
{
	int	i;

	for (i=0; dIndex[i].num != 0; i++)
	{
		InsertIndex(dIndex[i].num, 0);
	}

	InsertIndex(0, 0);
}

/**
 * Indexing by the Thread
 *
 * Arg: void
 * Ret: none
*/
void JkMemoBoard::ThreadIndex(void)
{
	int	iIndexOfdIndex;
	int	iThreadNum;
	int	iThreadL;
	int	iDepth;
	int	iExit;

	for (iIndexOfdIndex=0; dIndex[iIndexOfdIndex].num != 0; iIndexOfdIndex++)
	{
		if (dIndex[iIndexOfdIndex].thread != 0) continue;

		InsertIndex(dIndex[iIndexOfdIndex].num, 0);
		iThreadNum = dIndex[iIndexOfdIndex].num;
		iThreadL = iIndexOfdIndex;

		iExit = FALSE;

		do
		{
			iThreadL = FindThread(iThreadNum, iThreadL);

			if (iThreadL >= 0)
			{
				iDepth = PushIndex(iThreadNum, iThreadL);

				InsertIndex(dIndex[iThreadL].num, iDepth);
				iThreadNum = dIndex[iThreadL].num;
			}
			else
			{
				if (PopIndex(&iThreadNum, &iThreadL) == FALSE)
				{
					iExit = TRUE;
				}
			}
		} while (iExit == FALSE);
	}
	
	InsertIndex(dIndex[iIndexOfdIndex].num, 0);
}

/**
 * Find Thread
 *
 * Arg:
 *	@iNum: #, @iUpper: Upper #
 *
 * Ret:
 *	@i: #
*/
int JkMemoBoard::FindThread(int iNum, int iUpper)
{
	int	i;

	for (i=iUpper-1; i>=0; i--)
	{
		if (dIndex[i].thread == iNum) return i;
	}

	return -1;
}

/**
 * Insert Index
*/
int JkMemoBoard::InsertIndex(int iNum, int iDepth)
{
	static int lp = 0;

	tIndex[lp].num	 = iNum;
	tIndex[lp].depth = iDepth;
	lp++;

	return	lp;
}

/**
 * PushIndex
*/
int JkMemoBoard::PushIndex(int iNum, int i)
{
	if (sp > STACK_SIZE)	return	FALSE;

	stack.num1[sp] = iNum;
	stack.num2[sp] = i;
	sp++;

	return	sp;
}

/**
 * PopIndex
*/
int JkMemoBoard::PopIndex(int* iNum, int* i)
{
	if (sp == 0)
	{
		return	FALSE;
	}

	sp--;
	*iNum = stack.num1[sp];
	*i   = stack.num2[sp];

	return	TRUE;
}
	
/**
 * AdjustIndex
*/
void JkMemoBoard::AdjustIndex()
{
	int	i, j;

	/**
	 * DB Fetch
	*/
	for (i=0; dIndex[i].num != 0; i++)
	{
		if (dIndex[i].thread == 0)
		{
			continue;
		}

		/**
		 * Check wherether there is a mother thread.
		*/
		for (j=i+1; dIndex[j].num != 0; j++)
		{
			if (dIndex[j].num == dIndex[i].thread)
			{
				 break;
			}
		}

		if (dIndex[j].num == 0)
		{
			dIndex[i].thread = 0;
		}
	}
}

/**
 * Get the Article Information
 *
 * Arg:
 *	@iNum: #, @head: list
 * Ret:
 *	TRUE or FALSE
*/
int JkMemoBoard::GetArticleInfo(int iNum, DbList* head)
{
	/**
	 * Set to NULL for free()
	*/
	head->iNum = iNum;
	head->iAccess = DBiFetch("%d.iAccess", iNum);

	head->pszName = NULL;
	head->pszEMail = NULL;
	head->pszTitle = NULL;
	head->pszDate = NULL;
	head->pszTime = NULL;

	/**
	 * Fetch Data from DB.
	*/
	if ((head->pszName = qRemoveSpace(DBFetch("%d.pszName", iNum))) == NULL)
		return	FALSE;

	if ((head->pszEMail = qRemoveSpace(DBFetch("%d.pszEMail", iNum))) == NULL)
		return	FALSE;

	if ((head->pszTitle = qRemoveSpace(DBFetch("%d.pszTitle", iNum))) == NULL)
		return	FALSE;

	if ((head->pszIpAdd = qRemoveSpace(DBFetch("%d.pszIpAdd", iNum))) == NULL)
		return	FALSE;

	if ((head->pszDate = qRemoveSpace(DBFetch("%d.pszDate", iNum))) == NULL)
		return	FALSE;
	
	if ((head->pszTime = qRemoveSpace(DBFetch("%d.pszTime", iNum))) == NULL)
		return	FALSE;

	return	TRUE;
}

/**
 * Free Article Info JkMemory
 *
 * Arg:
 *	@head: list
 * Ret: none
*/
void JkMemoBoard::FreeArticleInfo(DbList* head)
{
	if (head->pszName != NULL)	free(head->pszName);
	if (head->pszEMail != NULL)	free(head->pszEMail);
	if (head->pszTitle != NULL)	free(head->pszTitle);
	if (head->pszDate != NULL)	free(head->pszDate);
	if (head->pszTime != NULL)	free(head->pszTime);

}

/**
 * Run
 */
void JkMemoBoard::Run()
{
	if (strcmp(m_pJkMemoMain->GetDbMode(), "Board") == 0)
	{
		ShowList();
	}
}
