/*
	JkMemoMain.cpp

	Author	: Kang Jul Ki (Master 2nd)
	E-mail	: jkkang@sel.chonbuk.ac.kr
	Homepage: http://sel.chonbuk.ac.kr/~jkkang/
	Date	: May 1, 2000
	Class	: Software Engineering Lab
		  Dept. of Electronic Engineering
		  Chonbuk National Univ.	

	Compiler: Visual C++ 6.0 & gcc 2.7.2
	Tab Size: 8
	
	Copyright (c) 2000, Kang Jul Ki
*/

#include "JkMemoMain.h"
#include "JkMemoBoard.h"
#include "JkMemoWrite.h"

#if defined(UNIX)
#include <crypt.h>
#else
#include "JkCrypt.h"
#endif

/**
 * Construtor.
 *
 * Arg: none
 * Ret: none
 */
JkMemoMain::JkMemoMain()
{
	m_pSysConfig	= new SysConfig;
	m_pDb		= new Db;
}

/**
 * Destructor.
 */
JkMemoMain::~JkMemoMain()
{
	delete m_pSysConfig;
	delete m_pDb;
}

/**
 * Get Sys Configuration.
 *
 * Arg: void
 * Ret: none
*/
void JkMemoMain::GetSysConfig(void)
{
	char	szPath[0xff] = "";

	Q_CGIenv	env;

	/**
	 * Fetch CGI Env
	*/
	qCGIenv(&env);

	/**
	 * Get IP Address
	*/
	if (env.remote_host == NULL)	// Domain Name has no.
	{
		m_pszIpAdd = env.remote_addr;
	}
	else
	{
		m_pszIpAdd = env.remote_host;
	}

	/**
	 * Get Date & Time
	*/
	sprintf(m_szDate, "%04d/%02d/%02d", env.year, env.mon, env.day);
	sprintf(m_szTime, "%02d:%02d", env.hour, env.min);

	/**
	 * Get DB Path
	*/
	sprintf(szPath, "./db/%s.gdbm", GetDbName());

	/**
	 * If dbName.gdbm is not been, Create it.
	*/
	if (qCheckFile(szPath) == 0)
	{
		if (DBCreate(szPath, 0666) != TRUE)
		{
			qError("DB(%s) can't be created.", szPath);
		}
		else
		{
			DBOpen(szPath, GDBM_REPLACE);

			DBiStore("SysConfig.iTotalCount", 0, GDBM_REPLACE);
			DBiStore("SysConfig.iTodayCount", 0, GDBM_REPLACE);
			DBStore("SysConfig.pszInitDate", GetDate(), GDBM_REPLACE);

			DBiStore("SysConfig.iLastNum", 0, GDBM_REPLACE);
			DBiStore("SysConfig.iWidth", 400, GDBM_REPLACE);
			DBiStore("SysConfig.iArticlePerPage", 10, GDBM_REPLACE);
			DBiStore("SysConfig.iIsPostAllow", ALLOW, GDBM_REPLACE);
			DBStore("SysConfig.pszAd", " Էϼ", GDBM_REPLACE);

#if defined(UNIX)
			DBStore("SysConfig.pszPasswd", crypt("jkkang65", "jk"), GDBM_REPLACE);
#else
			DBStore("SysConfig.pszPasswd", jk_crypt("jkkang65", "jk"), GDBM_REPLACE);
#endif
	
			DBClose();
		}
	}

	DBOpen(szPath, GDBM_REPLACE);

	/**
	* Get Configuration Information on the DB
	*/
	m_pSysConfig->iTotalCount = DBiFetch("SysConfig.iTotalCount");
	m_pSysConfig->iTodayCount = DBiFetch("SysConfig.iTodayCount");
	m_pSysConfig->pszInitDate = DBFetch("SysConfig.pszInitDate");
	m_pSysConfig->iLastNum = DBiFetch("SysConfig.iLastNum");
	m_pSysConfig->iWidth = DBiFetch("SysConfig.iWidth");
	m_pSysConfig->iArticlePerPage = DBiFetch("SysConfig.iArticlePerPage");
	m_pSysConfig->iIsPostAllow = DBiFetch("SysConfig.iIsPostAllow");
	m_pSysConfig->pszAd = DBFetch("SysConfig.pszAd");
	m_pSysConfig->pszPasswd = DBFetch("SysConfig.pszPasswd");

	/**
	 * Add Visit Count
	*/
	DBiStore("SysConfig.iTotalCount", GetTotalCount()+1, GDBM_REPLACE);
	DBiStore("SysConfig.iTodayCount", GetTodayCount()+1, GDBM_REPLACE);		

	if (strcmp(GetInitDate(), GetDate()) != 0)
	{
		DBiStore("SysConfig.iTodayCount", 0, GDBM_REPLACE);
		DBStore("SysConfig.pszInitDate", GetDate(), GDBM_REPLACE);	
	}

	DBClose();
}

/**
 * Get DB Configuration.
 *
 * Arg: void
 * Ret: none
*/
void JkMemoMain::GetDbConfig(void)
{
	/**
	 * Check dbName?
	*/
	if ((m_pDb->pDbName = qValue("dbName")) == NULL)
	{
		qError("DB ̸ ؾ մϴ. (e.g: JkJkMemo.cgi?dbName≠)");
	}
	if (strcmp(m_pDb->pDbName, "") == 0)
	{
		qError("dbName ؾ մϴ.");
	}

	/**
	 * Check dbMode?
	*/
	m_pDb->pDbMode = qValueDefault("Board", "dbMode");
	if (strcmp(m_pDb->pDbMode, "") == 0)
	{
		m_pDb->pDbMode = "Board";
	}

	/**
	 * Check dBPage?
	*/
	if ((m_pDb->iPage = qiValue("dbPage")) == 0)
	{
		m_pDb->iPage = 1;
	}

	/**
	 * Check dbNum?
	*/
	m_pDb->iNum  = qiValue("dbNum");
}

/**
 * Print Head Html.
 *
 * Arg:
 *	@szTitle: The Title of the Web Browser
 * Ret: none
*/
void JkMemoMain::HeadHtml(char* szTitle)
{
	/**
	 * Define mime type.
	*/
	qContentType("text/html");

	/**
	 * Program Information.
	*/
	ProgramInfo();

	/**
	 * Head
	*/
	printf("<html>\n");
	printf("<head>\n");
	printf("  <title>%s</title>\n", szTitle);
	printf("    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=euc-kr\">\n");

	if (GetRefreshValue() != 0)
	{
		printf("    <meta http-equiv=\"refresh\" content=\"%d; url=%s?dbName=%s\">\n", GetRefreshValue(), qCGIname(), GetDbName());
	}

	printf("</head>\n");

	printf("\n<!-- Style Sheet -->\n");
	printf("<style type='text/css'>\n");
	printf("<--\n");
	printf(" A:link		{ color:#92aa9d;text-decoration:none; }\n");
	printf(" A:visited	{ color:#92aa9d;text-decoration:none; }\n");
	printf(" A:active	{ color:#92aa9d;text-decoration:none; }\n");	
	printf(" A:hover	{ color:#92aa9d;text-decoration:none; }\n");
	printf(" .a		{ text-decoration:none; }\n");
	printf(" .fo		{ background-color:#f5fff9; border:1 solid black; }\n");
	printf(" .bl		{ background-color:#ffffff; border:1 solid black; }\n");
	printf(" .small		{ font-family:; font-size:8pt;	}\n");
	printf(" .norm		{ font-family:; font-size:9pt;	}\n");
	printf(" .big		{ font-family:; font-size:14pt;	}\n");
	printf(" .editbox	{ background-color:#ffffff; border:1 solid black; }\n");
	printf(" .button	{ height:20px; background-color:#ffffff; font-size:9pt; border:1 solid black; }\n");
	printf(" select		{ background-color:white; }\n");
	printf(" p,br,body,table,td,select,input,form,textarea,option { font-family:; font-size:9pt; }\n");
	printf("-->\n");
	printf("</style>\n");
	printf("\n");

	printf("<body bgcolor='white'>\n\n");
	printf("<center>\n");
}

/**
 * Print Tail Html.
 *
 * Arg: void
 * Ret: none
*/
void JkMemoMain::TailHtml(void)
{
	printf("</center>\n\n");
	printf("</body>\n");
	printf("</html>\n");
}

/**
 * Refresh the current page after the time.
 *
 * Arg: void
 * Ret: none
*/
void JkMemoMain::RefreshList(void)
{
	qContentType("text/html");

	printf("<html>\n");
	printf("<head>\n");
	printf("  <title>Wait a moment while refreshing this page.</title>\n");
	printf("  <meta http-equiv=\"refresh\" content=\"0; url='%s?dbName=%s'>\"\n", qCGIname(), GetDbName());
	printf("</head>\n");
	printf("</html>\n");

	/* Running Time */
	runTime = clock();  /* microsec */
	runTime /= 1000;    /* milisec */
	printf("\n<!-- CPU Time %.7f seconds -->\n", (float)runTime/(float)1000);

	qFree();
	exit(1);
}

/**
 * Developing Info
 *
 * Arg: void
 * Ret: none
*/
void JkMemoMain::ProgramInfo(void)
{
	printf("<!-------------------------------------------------------------->\n");
	printf("<!--                   JK ޸ Ver %s                    -->\n", GetBuildInfo());
	printf("<!--                                                          -->\n");
	printf("<!--   Distribution site : http://sel.chonbuk.ac.kr/~jkkang/  -->\n");
	printf("<!--            Technical Info    : jkkang65@hanmail.net      -->\n");
	printf("<!--                                                          -->\n");
	printf("<!--                                Developed by Kang Jul Ki  -->\n");
	printf("<!--                                                          -->\n");
	printf("<!--        Copyleft (c) 1997-2000, Software Engineering Lab  -->\n");
	printf("<!--                         Dept. of Electronic Engineering  -->\n");
	printf("<!--                                  Chonbuk National Univ.  -->\n");
	printf("<!-------------------------------------------------------------->\n\n\n");

}
/**
 * Run the main function.
 *
 * Arg: void
 * Ret: none
 */
void JkMemoMain::Run(void)
{
	/**
	 * Start timer.
	*/
	clock();

	/**
	 * Fetch Query.
	*/
	qDecoder();

	/**
	 * Get Db Config.
	*/
	GetDbConfig();

	/**
	 * Get Sys Config.
	*/
	GetSysConfig();

	/**
	 * Which Mode?
	 */
	if (strcmp(GetDbMode(), "Board") == 0)
	{
		JkMemoBoard	MyJkMemoBoard(this);

		MyJkMemoBoard.Run();
	}
	else if (strcmp(GetDbMode(), "Write") == 0 || strcmp(GetDbMode(), "Remove") == 0 || strcmp(GetDbMode(), "RemoveSave") == 0)
	{
		JkMemoWrite	MyJkMemoWrite(this);

		MyJkMemoWrite.Run();
	}
	else if (strcmp(GetDbMode(), "AdminLogin") == 0|| strcmp(GetDbMode(), "AdminForm") || !strcmp(GetDbMode(), "AdminSave") == 0)
	{
		JkMemoWrite	MyJkMemoWrite(this);

		MyJkMemoWrite.Run();
	}

	/* Running Time */
	runTime = clock();  /* microsec */
	runTime /= 1000;    /* milisec */
	printf("\n<!-- CPU Time %.7f seconds -->\n", (float)runTime/(float)1000);

	qFree();
}

/**
 * The entry of program...
 */
int main(void)
{
	JkMemoMain	MyJkMemoMain;

	MyJkMemoMain.Run();

	return	1;
}
